"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const amqp_client_1 = require("@jfrog-pipelines/amqp-client");
const internal_clients_1 = require("@jfrog-pipelines/internal-clients");
const logger_1 = require("@jfrog-pipelines/logger");
const NodeService_1 = require("../node/NodeService");
const NodePoolService_1 = require("../nodePool/NodePoolService");
const BuildPlaneService_1 = require("../buildPlane/BuildPlaneService");
const NodeProvisioner_1 = require("../NodeProvisioner");
const NodeProvisionerTestCases_1 = require("./NodeProvisionerTestCases");
jest.mock('@jfrog/access-nodejs-client');
jest.mock('@jfrog-pipelines/configuration', () => {
    return {
        ConfigFactory: {
            config: {
                buildPlaneVersion: 'testversion'
            }
        }
    };
});
jest.mock('@jfrog-pipelines/amqp-client');
jest.mock('@jfrog-pipelines/logger', () => {
    const actualTracker = jest.requireActual('@jfrog-pipelines/logger');
    return {
        ...actualTracker,
        LoggerFactory: {
            getLogger: jest.fn().mockReturnValue({
                info: jest.fn(),
                error: jest.fn(),
                debug: jest.fn(),
                trace: jest.fn(),
                warn: jest.fn()
            }),
            configure: jest.fn()
        }
    };
});
jest.spyOn(internal_clients_1.AccessClientFactory, 'getClient').mockReturnValue({
    getNodeAccessToken: jest.fn().mockReturnValue('token')
});
const client = new amqp_client_1.AmqpClient('', '', { amqpUrl: '' }, logger_1.LoggerFactory.getLogger('NodeServiceTest'));
const publisher = new amqp_client_1.AmqpPublisher(client, '', amqp_client_1.publisherOptions, logger_1.LoggerFactory.getLogger('NodeServiceTest'));
const publishMessageMock = jest.spyOn(amqp_client_1.AmqpPublisher.prototype, 'publishMessage').mockResolvedValueOnce({});
describe('Test NodeProvisioner #provision', () => {
    const nodeProvisioner = new NodeProvisioner_1.NodeProvisioner(publisher);
    const nodePool = { id: 1, name: 'testpool', projectId: 1 };
    const getNodePoolMock = jest.spyOn(NodePoolService_1.NodePoolService.prototype, 'getNodePool').mockResolvedValue(nodePool);
    afterEach(() => {
        publishMessageMock.mockClear();
        getNodePoolMock.mockClear();
    });
    it('should not provision nodes based on build plane message count', async () => {
        const nodes = [
            { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.waiting },
            { friendlyName: 'test1', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.cached },
            { friendlyName: 'test2', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.success }
        ];
        const getMessageCountMock = jest
            .spyOn(BuildPlaneService_1.BuildPlaneService.prototype, 'getReadyMessageCount')
            .mockResolvedValueOnce(2);
        const getRunningNodesMock = jest.spyOn(NodeService_1.NodeService.prototype, 'getRunningNodes').mockResolvedValueOnce(nodes);
        const filterActiveNodesMock = jest
            .spyOn(NodeService_1.NodeService.prototype, 'filterActiveNodes')
            .mockImplementationOnce((nodes) => nodes.filter((node) => {
            return node.statusCode != NodeService_1.NodeStatus.cached;
        }));
        jest.spyOn(NodeService_1.NodeService.prototype, 'filterCachedNodes').mockImplementationOnce((nodes) => nodes.filter((node) => {
            return node.statusCode === NodeService_1.NodeStatus.cached;
        }));
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.waiting };
        const createNodeMock = jest.spyOn(NodeService_1.NodeService.prototype, 'createNode').mockResolvedValueOnce(node);
        const provisionResponse = await nodeProvisioner.provision(nodePool);
        await expect(provisionResponse).toBe(undefined);
        getMessageCountMock.mockClear();
        getRunningNodesMock.mockClear();
        filterActiveNodesMock.mockClear();
        createNodeMock.mockClear();
    });
    it('should provision nodes based on build plane message count', async () => {
        const nodes = [
            { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.waiting },
            { friendlyName: 'test1', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.cached },
            { friendlyName: 'test2', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.success }
        ];
        const getMessageCountMock = jest
            .spyOn(BuildPlaneService_1.BuildPlaneService.prototype, 'getReadyMessageCount')
            .mockResolvedValueOnce(5);
        const getRunningNodesMock = jest.spyOn(NodeService_1.NodeService.prototype, 'getRunningNodes').mockResolvedValueOnce(nodes);
        const filterActiveNodesMock = jest
            .spyOn(NodeService_1.NodeService.prototype, 'filterActiveNodes')
            .mockImplementationOnce((nodes) => nodes.filter((node) => {
            return node.statusCode != NodeService_1.NodeStatus.cached;
        }));
        jest.spyOn(NodeService_1.NodeService.prototype, 'filterCachedNodes').mockImplementationOnce((nodes) => nodes.filter((node) => {
            return node.statusCode === NodeService_1.NodeStatus.cached;
        }));
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.waiting };
        const createNodeMock = jest.spyOn(NodeService_1.NodeService.prototype, 'createNode').mockResolvedValueOnce(node);
        const provisionResponse = await nodeProvisioner.provision(nodePool);
        await expect(provisionResponse).toBe(undefined);
        getMessageCountMock.mockClear();
        getRunningNodesMock.mockClear();
        filterActiveNodesMock.mockClear();
        createNodeMock.mockClear();
    });
    it('should start cached nodes and create nodes based on buildPlaneMessage count', async () => {
        const nodePool = { id: 1, name: 'testpool', projectId: 1, numberOfNodes: 5 };
        const nodes = [{ friendlyName: 'test2', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.cached }];
        const getMessageCountMock = jest
            .spyOn(BuildPlaneService_1.BuildPlaneService.prototype, 'getReadyMessageCount')
            .mockResolvedValueOnce(5);
        const applyNodeWaitingStateSpy = jest
            .spyOn(NodeService_1.NodeService.prototype, 'applyNodeWaitingState')
            .mockResolvedValueOnce(nodes[0]);
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.waiting };
        const createNodeMock = jest.spyOn(NodeService_1.NodeService.prototype, 'createNode').mockResolvedValue(node);
        const getRunningNodesMock = jest.spyOn(NodeService_1.NodeService.prototype, 'getRunningNodes').mockResolvedValue(nodes);
        const provisionResponse = await nodeProvisioner.provision(nodePool);
        await expect(provisionResponse).toBe(undefined);
        expect(applyNodeWaitingStateSpy).toHaveBeenCalledTimes(1);
        expect(createNodeMock).toHaveBeenCalledTimes(4);
        getMessageCountMock.mockClear();
        getRunningNodesMock.mockClear();
    });
    it.each((0, NodeProvisionerTestCases_1.getProvisionerTestCases)())('should create %s nodes and start %s nodes when given nodeLimit is %s, %s bpMsgs, %s success nodes, %s waiting nodes, %s cached, %s processing, %s failure nodes, %s stopped nodes', async (nodeCreated, nodeStarted, nodeLimit, buildPlaneMessageCount, successNodes, waitingNodes, cachedNodes, processingNodes, failureNodes, stoppedNodes) => {
        const nodePool = { id: 1, name: 'testpool', projectId: 1, numberOfNodes: nodeLimit };
        const nodes = (0, NodeProvisionerTestCases_1.getMockNodes)({
            successNodes,
            waitingNodes,
            cachedNodes,
            processingNodes,
            failureNodes,
            stoppedNodes
        });
        const getMessageCountMock = jest
            .spyOn(BuildPlaneService_1.BuildPlaneService.prototype, 'getReadyMessageCount')
            .mockResolvedValue(buildPlaneMessageCount);
        const applyNodeWaitingStateSpy = jest
            .spyOn(NodeService_1.NodeService.prototype, 'applyNodeWaitingState')
            .mockResolvedValue(nodes[0]);
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.waiting };
        const createNodeMock = jest.spyOn(NodeService_1.NodeService.prototype, 'createNode').mockResolvedValue(node);
        const getRunningNodesMock = jest.spyOn(NodeService_1.NodeService.prototype, 'getRunningNodes').mockResolvedValue(nodes);
        const provisionResponse = await nodeProvisioner.provision(nodePool);
        await expect(provisionResponse).toBe(undefined);
        expect(createNodeMock).toHaveBeenCalledTimes(nodeCreated);
        expect(applyNodeWaitingStateSpy).toHaveBeenCalledTimes(nodeStarted);
        getMessageCountMock.mockClear();
        getRunningNodesMock.mockClear();
    });
});
//# sourceMappingURL=NodeProvisioner.test.js.map