"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NodeService = exports.NodeActions = exports.nodeAccessorID = exports.idleNodeStatuses = exports.unhealthyNodeStatuses = exports.activeNodeStatus = exports.NodeStatus = void 0;
const node_util_1 = require("node:util");
const data_access_layer_1 = require("@jfrog-pipelines/data-access-layer");
const internal_clients_1 = require("@jfrog-pipelines/internal-clients");
const logger_1 = require("@jfrog-pipelines/logger");
const NodeDao_1 = require("./NodeDao");
var NodeStatus;
(function (NodeStatus) {
    NodeStatus[NodeStatus["queued"] = 4000] = "queued";
    NodeStatus[NodeStatus["processing"] = 4001] = "processing";
    NodeStatus[NodeStatus["success"] = 4002] = "success";
    NodeStatus[NodeStatus["failure"] = 4003] = "failure";
    NodeStatus[NodeStatus["waiting"] = 4005] = "waiting";
    NodeStatus[NodeStatus["cancelled"] = 4006] = "cancelled";
    NodeStatus[NodeStatus["unstable"] = 4007] = "unstable";
    NodeStatus[NodeStatus["skipped"] = 4008] = "skipped";
    NodeStatus[NodeStatus["timeout"] = 4009] = "timeout";
    NodeStatus[NodeStatus["stopped"] = 4010] = "stopped";
    NodeStatus[NodeStatus["deleted"] = 4011] = "deleted";
    NodeStatus[NodeStatus["cached"] = 4012] = "cached";
    NodeStatus[NodeStatus["cancelling"] = 4013] = "cancelling";
    NodeStatus[NodeStatus["timingout"] = 4014] = "timingout";
})(NodeStatus = exports.NodeStatus || (exports.NodeStatus = {}));
const ONE_MINUTE_MILLIS = 60000;
exports.activeNodeStatus = [
    NodeStatus.success,
    NodeStatus.waiting,
    NodeStatus.stopped,
    NodeStatus.failure,
    NodeStatus.queued,
    NodeStatus.processing,
    NodeStatus.cached
];
exports.unhealthyNodeStatuses = [NodeStatus.failure, NodeStatus.deleted, NodeStatus.stopped];
exports.idleNodeStatuses = [NodeStatus.waiting, NodeStatus.success, NodeStatus.cached];
exports.nodeAccessorID = '1';
var NodeActions;
(function (NodeActions) {
    NodeActions[NodeActions["CREATE"] = 10] = "CREATE";
    NodeActions[NodeActions["CACHE"] = 20] = "CACHE";
    NodeActions[NodeActions["START"] = 30] = "START";
    NodeActions[NodeActions["DELETE"] = 40] = "DELETE";
})(NodeActions = exports.NodeActions || (exports.NodeActions = {}));
class NodeService {
    constructor(publisher) {
        this.publisher = publisher;
        this.logger = logger_1.LoggerFactory.getLogger('NodeService');
        this.nodeDao = new NodeDao_1.NodeDao();
        this.accessClient = internal_clients_1.AccessClientFactory.getClient();
    }
    async createNode(attribute) {
        const node = await this.nodeDao.create(attribute);
        this.logger.debug(`Node with node pool id ${attribute.nodePoolId} created successfully with name ${node.friendlyName}`);
        const nodeToken = await this.getNodeAccessToken(node);
        this.logger.trace(`Succesfully created nodeToken for ${node.friendlyName}`);
        await this.updateNodeBootStrapToken(node, nodeToken);
        await this.publisher.publishMessage('core.marshaller', { action: NodeActions.CREATE, nodeId: node.id });
        this.logger.trace(`Succesfully published message to marshaller to provision node for ${node.friendlyName}`);
        await this.sendWWWSignal(node);
        return node;
    }
    async getRunningNodes(poolID) {
        const queryBuilder = new data_access_layer_1.SequelizeQueryBuilder({})
            .where(`nodePoolId`, poolID)
            .where('statusCode', exports.activeNodeStatus)
            .where('stopRequested', 'false');
        return this.nodeDao.findAll(queryBuilder.build());
    }
    async getAllNodes(poolID) {
        const queryBuilder = new data_access_layer_1.SequelizeQueryBuilder({}).where(`nodePoolId`, poolID);
        return this.nodeDao.findAll(queryBuilder.build());
    }
    async getOrphanNodes() {
        const queryBuilder = new data_access_layer_1.SequelizeQueryBuilder({}).where(`nodePoolId`, null);
        return this.nodeDao.findAll(queryBuilder.build());
    }
    filterActiveNodes(nodes) {
        const activeNodeStatuses = new Set([
            NodeStatus.waiting,
            NodeStatus.success,
            NodeStatus.processing,
            NodeStatus.cached
        ]);
        const activeNodes = nodes.filter((node) => {
            return activeNodeStatuses.has(node.statusCode);
        });
        return activeNodes;
    }
    filterAvailableNodes(nodes) {
        const runningNodesStatuses = new Set([NodeStatus.waiting, NodeStatus.success]);
        const activeNodes = nodes.filter((node) => {
            return runningNodesStatuses.has(node.statusCode);
        });
        return activeNodes;
    }
    filterIdleNodes(nodePool, nodes) {
        return nodes.filter((node) => {
            return this.isNodeIdle(nodePool, node);
        });
    }
    filterCachedNodes(nodes) {
        const cachedNodes = nodes.filter((node) => {
            return node.statusCode === NodeStatus.cached;
        });
        return cachedNodes;
    }
    async applyNodeToStoppedState(node) {
        this.logger.trace(`Stopping the node ${node.id}:${node.friendlyName}`);
        return this.applyNodeState(node, NodeActions.DELETE, NodeStatus.stopped);
    }
    async applyNodeToCachedState(node) {
        this.logger.trace(`Caching the node ${node.id}:${node.friendlyName}`);
        return this.applyNodeState(node, NodeActions.CACHE, NodeStatus.cached);
    }
    async applyNodeWaitingState(node) {
        this.logger.trace(`Starting the cache node ${node.id}:${node.friendlyName}`);
        return this.applyNodeState(node, NodeActions.START, NodeStatus.waiting);
    }
    isNodeIdle(nodePool, node) {
        if (node.stopRequested) {
            return true;
        }
        if (exports.unhealthyNodeStatuses.includes(node.statusCode)) {
            return true;
        }
        const nodeLastUpdatedAt = new Date(node.statusLastUpdatedAt).getTime();
        return (exports.idleNodeStatuses.includes(node.statusCode) &&
            Date.now() - nodeLastUpdatedAt >= nodePool.nodeIdleIntervalInMins * ONE_MINUTE_MILLIS);
    }
    async applyNodeState(node, action, status) {
        try {
            this.logger.trace(`Updating the node ${node.id}:${node.friendlyName} with status: ${status}`);
            const updateResponse = await this.nodeDao.updateByInstance(node, {
                statusCode: status,
                statusLastUpdatedAt: new Date()
            });
            await this.publisher.publishMessage('core.marshaller', {
                action: action,
                projectId: node.projectId,
                nodeId: node.id
            });
            this.logger.trace(`Succesfully published message to marshaller for node ${node.id}:${node.friendlyName} with action: ${action}`);
            return updateResponse;
        }
        catch (error) {
            this.logger.warn(`Failed to applyNodeState to ${status} for node id ${node.id} due to err ${(0, node_util_1.inspect)(error)} `);
        }
        return node;
    }
    async getNodeAccessToken(node) {
        return this.accessClient.getNodeAccessToken(node);
    }
    async updateNodeBootStrapToken(node, token) {
        return this.nodeDao.updateByInstance(node, { systemPropertyBag: { token: token } });
    }
    async sendWWWSignal(node) {
        const signalPayload = {
            eventType: `node.created`,
            rooms: `v3/projects/${node.projectId}`,
            payload: { id: node.id }
        };
        return this.publisher.publishMessage(`www.signals`, signalPayload);
    }
}
exports.NodeService = NodeService;
//# sourceMappingURL=NodeService.js.map